'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.defineRecordProperties = defineRecordProperties;
exports.makeCursor = makeCursor;
exports.listToKeyPath = listToKeyPath;
exports.newKeyPath = newKeyPath;
exports.valToKeyPath = valToKeyPath;
exports.subCursor = subCursor;
exports.updateCursor = updateCursor;
exports.wrappedValue = wrappedValue;

var _immutable = require('immutable');

var _keyed = require('./keyed');

var _keyed2 = _interopRequireDefault(_keyed);

var _indexed = require('./indexed');

var _indexed2 = _interopRequireDefault(_indexed);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function setProp(prototype, name) {
  Object.defineProperty(prototype, name, {
    get: function get() {
      return this.get(name);
    },
    set: function set() {
      if (!this.__ownerID) {
        throw new Error('Cannot set on an immutable record.');
      }
    }
  });
}

function defineRecordProperties(cursor, value) {
  try {
    value._keys.forEach(setProp.bind(undefined, cursor));
  } catch (error) {
    // Object.defineProperty failed. Probably IE8.
  }
}

function makeCursor(rootData, keyPath, updater, deref, value) {
  if (arguments.length < 5) {
    value = rootData.getIn(keyPath);
  }
  var size = value && value.size;
  var Cursor = _immutable.Iterable.isIndexed(value) ? _indexed2.default : _keyed2.default;
  var cursor = new Cursor(rootData, keyPath, updater, deref, size);

  if (value instanceof _immutable.Record) {
    defineRecordProperties(cursor, value);
  }

  return cursor;
}

function listToKeyPath(list) {
  return Array.isArray(list) ? list : (0, _immutable.Iterable)(list).toArray();
}

function newKeyPath(head, tail) {
  return head.concat(listToKeyPath(tail));
}

function valToKeyPath(val) {
  return Array.isArray(val) ? val : _immutable.Iterable.isIterable(val) ? val.toArray() : [val];
}

function subCursor(cursor, keyPath, value) {
  if (arguments.length < 3) {
    return makeCursor( // call without value
    cursor._rootData, newKeyPath(cursor._keyPath, keyPath), cursor._updater, cursor._deref);
  }
  return makeCursor(cursor._rootData, newKeyPath(cursor._keyPath, keyPath), cursor._updater, cursor._deref, value);
}

function updateCursor(cursor, changeFn) {
  var deepChange = arguments.length > 2;
  var updateFn = function updateFn(oldState) {
    return oldState.updateIn(cursor._keyPath, deepChange ? (0, _immutable.Map)() : undefined, changeFn);
  };
  return makeCursor(cursor._updater(updateFn), cursor._keyPath, cursor._updater, cursor._deref);
}

function wrappedValue(cursor, keyPath, value) {
  return _immutable.Iterable.isIterable(value) ? subCursor(cursor, keyPath, value) : value;
}